@@ -1,6 +1,9 @@
 /*
  * compat.h
- * Copyright (c) 2ndQuadrant, 2010-2017
+ * Copyright (c) 2ndQuadrant, 2010-2018
+ *
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1994, Regents of the University of California
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,10 +23,8 @@
 #ifndef _COMPAT_H_
 #define _COMPAT_H_
 
-extern void
-appendConnStrVal(PQExpBuffer buf, const char *str);
+extern void appendConnStrVal(PQExpBuffer buf, const char *str);
 
-extern void
-appendShellString(PQExpBuffer buf, const char *str);
+extern void appendShellString(PQExpBuffer buf, const char *str);
 
 #endif
