@@ -30,10 +30,15 @@
  * NODE STATUS
  * NODE CHECK
  *
+ * DAEMON STATUS
+ * DAEMON PAUSE
+ * DAEMON UNPAUSE
+ *
  * For internal use:
  * NODE REJOIN
  * NODE SERVICE
  *
+ *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -62,6 +67,7 @@
 #include "repmgr-action-bdr.h"
 #include "repmgr-action-node.h"
 #include "repmgr-action-cluster.h"
+#include "repmgr-action-daemon.h"
 
 #include <storage/fd.h>			/* for PG_TEMP_FILE_PREFIX */
 
@@ -98,7 +104,7 @@ main(int argc, char **argv)
 {
 	t_conninfo_param_list default_conninfo = T_CONNINFO_PARAM_LIST_INITIALIZER;
 
-	int			optindex;
+	int			optindex = 0;
 	int			c;
 
 	char	   *repmgr_command = NULL;
@@ -108,6 +114,7 @@ main(int argc, char **argv)
 	char	   *dummy_action = "";
 
 	bool		help_option = false;
+	bool		option_error_found = false;
 
 	set_progname(argv[0]);
 
@@ -178,7 +185,10 @@ main(int argc, char **argv)
 		strncpy(runtime_options.username, pw->pw_name, MAXLEN);
 	}
 
-	while ((c = getopt_long(argc, argv, "?Vb:f:FwWd:h:p:U:R:S:D:ck:L:tvC:", long_options,
+	/* Make getopt emitting errors */
+	opterr = 1;
+
+	while ((c = getopt_long(argc, argv, "?Vb:f:FwWd:h:p:U:R:S:D:ck:L:qtvC:", long_options,
 							&optindex)) != -1)
 	{
 		/*
@@ -196,13 +206,7 @@ main(int argc, char **argv)
 			case OPT_HELP:		/* --help */
 				help_option = true;
 				break;
-			case '?':
-				/* Actual help option given */
-				if (strcmp(argv[optind - 1], "-?") == 0)
-				{
-					help_option = true;
-				}
-				break;
+
 			case 'V':
 
 				/*
@@ -440,6 +444,10 @@ main(int argc, char **argv)
 				runtime_options.siblings_follow = true;
 				break;
 
+			case OPT_REPMGRD_NO_PAUSE:
+				runtime_options.repmgrd_no_pause = true;
+				break;
+
 				/*----------------------
 				 * "node status" options
 				 *----------------------
@@ -473,6 +481,10 @@ main(int argc, char **argv)
 				runtime_options.slots = true;
 				break;
 
+			case OPT_MISSING_SLOTS:
+				runtime_options.missing_slots = true;
+				break;
+
 			case OPT_HAS_PASSFILE:
 				runtime_options.has_passfile = true;
 				break;
@@ -572,6 +584,12 @@ main(int argc, char **argv)
 				logger_output_mode = OM_DAEMON;
 				break;
 
+
+				/* --quiet */
+			case 'q':
+				runtime_options.quiet = true;
+				break;
+
 				/* --terse */
 			case 't':
 				runtime_options.terse = true;
@@ -627,9 +645,24 @@ main(int argc, char **argv)
 								 _("--recovery-min-apply-delay is now a configuration file parameter, \"recovery_min_apply_delay\""));
 				break;
 
+			case ':':   /* missing option argument */
+				option_error_found = true;
+				break;
+			case '?':
+				/* Actual help option given? */
+				if (strcmp(argv[optind - 1], "-?") == 0)
+				{
+					help_option = true;
+					break;
+				}
+				/* otherwise fall through to default */
+			default:    /* invalid option */
+				option_error_found = true;
+				break;
 		}
 	}
 
+
 	/*
 	 * If -d/--dbname appears to be a conninfo string, validate by attempting
 	 * to parse it (and if successful, store the parsed parameters)
@@ -730,9 +763,10 @@ main(int argc, char **argv)
 	if (cli_errors.head != NULL)
 	{
 		free_conninfo_params(&source_conninfo);
-		exit_with_cli_errors(&cli_errors);
+		exit_with_cli_errors(&cli_errors, NULL);
 	}
 
+
 	/*----------
 	 * Determine the node type and action; following are valid:
 	 *
@@ -876,6 +910,21 @@ main(int argc, char **argv)
 			else if (strcasecmp(repmgr_action, "CLEANUP") == 0)
 				action = CLUSTER_CLEANUP;
 		}
+		else if (strcasecmp(repmgr_command, "DAEMON") == 0)
+		{
+			if (help_option == true)
+			{
+				do_daemon_help();
+				exit(SUCCESS);
+			}
+
+			if (strcasecmp(repmgr_action, "STATUS") == 0)
+				action = DAEMON_STATUS;
+			else if (strcasecmp(repmgr_action, "PAUSE") == 0)
+				action = DAEMON_PAUSE;
+			else if (strcasecmp(repmgr_action, "UNPAUSE") == 0)
+				action = DAEMON_UNPAUSE;
+		}
 		else
 		{
 			valid_repmgr_command_found = false;
@@ -979,9 +1028,30 @@ main(int argc, char **argv)
 	if (cli_errors.head != NULL)
 	{
 		free_conninfo_params(&source_conninfo);
-		exit_with_cli_errors(&cli_errors);
+
+		exit_with_cli_errors(&cli_errors, valid_repmgr_command_found == true ? repmgr_command : NULL);
 	}
 
+	/* no errors detected by repmgr, but getopt might have */
+	if (option_error_found == true)
+	{
+		if (valid_repmgr_command_found == true)
+		{
+			printf(_("Try \"%s --help\" or \"%s %s --help\" for more information.\n"),
+				   progname(),
+				   progname(),
+				   repmgr_command);
+		}
+		else
+		{
+			printf(_("Try \"repmgr --help\" for more information.\n"));
+		}
+
+		free_conninfo_params(&source_conninfo);
+		exit(ERR_BAD_CONFIG);
+	}
+
+
 	/*
 	 * Print any warnings about inappropriate command line options, unless
 	 * -t/--terse set
@@ -1077,6 +1147,17 @@ main(int argc, char **argv)
 		logger_set_min_level(LOG_INFO);
 	}
 
+	/*
+	 * If -q/--quiet supplied, suppress any non-ERROR log output.
+	 * This overrides everything else; we'll leave it up to the user to deal with the
+	 * consequences of e.g. running --dry-run together with -q/--quiet.
+	 */
+	if (runtime_options.quiet == true)
+	{
+		logger_set_level(LOG_ERROR);
+	}
+
+
 
 	/*
 	 * Node configuration information is not needed for all actions, with
@@ -1242,6 +1323,17 @@ main(int argc, char **argv)
 			do_cluster_cleanup();
 			break;
 
+			/* DAEMON */
+		case DAEMON_STATUS:
+			do_daemon_status();
+			break;
+		case DAEMON_PAUSE:
+			do_daemon_pause();
+			break;
+		case DAEMON_UNPAUSE:
+			do_daemon_unpause();
+			break;
+
 		default:
 			/* An action will have been determined by this point  */
 			break;
@@ -1463,6 +1555,8 @@ check_cli_parameters(const int action)
 		{
 			case PRIMARY_UNREGISTER:
 			case STANDBY_UNREGISTER:
+			case WITNESS_UNREGISTER:
+			case CLUSTER_CLEANUP:
 			case CLUSTER_EVENT:
 			case CLUSTER_MATRIX:
 			case CLUSTER_CROSSCHECK:
@@ -1503,6 +1597,7 @@ check_cli_parameters(const int action)
 			case STANDBY_CLONE:
 			case STANDBY_REGISTER:
 			case STANDBY_FOLLOW:
+			case BDR_REGISTER:
 				break;
 			default:
 				item_list_append_format(&cli_warnings,
@@ -1685,6 +1780,18 @@ check_cli_parameters(const int action)
 		}
 	}
 
+	if (runtime_options.repmgrd_no_pause == true)
+	{
+		switch (action)
+		{
+			case STANDBY_SWITCHOVER:
+				break;
+			default:
+				item_list_append_format(&cli_warnings,
+										_("--repmgrd-no-pause will be ignored when executing %s"),
+										action_name(action));
+		}
+	}
 
 	if (runtime_options.config_files[0] != '\0')
 	{
@@ -1713,6 +1820,8 @@ check_cli_parameters(const int action)
 			case WITNESS_UNREGISTER:
 			case NODE_REJOIN:
 			case NODE_SERVICE:
+			case DAEMON_PAUSE:
+			case DAEMON_UNPAUSE:
 				break;
 			default:
 				item_list_append_format(&cli_warnings,
@@ -1792,6 +1901,14 @@ action_name(const int action)
 			return "CLUSTER MATRIX";
 		case CLUSTER_CROSSCHECK:
 			return "CLUSTER CROSSCHECK";
+
+		case DAEMON_STATUS:
+			return "DAEMON STATUS";
+		case DAEMON_PAUSE:
+			return "DAEMON PAUSE";
+		case DAEMON_UNPAUSE:
+			return "DAEMON UNPAUSE";
+
 	}
 
 	return "UNKNOWN ACTION";
@@ -1819,6 +1936,42 @@ print_error_list(ItemList *error_list, int log_level)
 }
 
 
+void
+print_status_header(int cols, ColHeader *headers)
+{
+	int i;
+
+	for (i = 0; i < cols; i++)
+	{
+		if (i == 0)
+			printf(" ");
+		else
+			printf(" | ");
+
+		printf("%-*s",
+			   headers[i].max_length,
+			   headers[i].title);
+	}
+	printf("\n");
+	printf("-");
+
+	for (i = 0; i < cols; i++)
+	{
+		int			j;
+
+		for (j = 0; j < headers[i].max_length; j++)
+			printf("-");
+
+		if (i < (cols - 1))
+			printf("-+-");
+		else
+			printf("-");
+	}
+
+	printf("\n");
+}
+
+
 void
 print_help_header(void)
 {
@@ -1845,8 +1998,9 @@ do_help(void)
 	printf(_("    %s [OPTIONS] standby {register|unregister|clone|promote|follow|switchover}\n"), progname());
 	printf(_("    %s [OPTIONS] bdr     {register|unregister}\n"), progname());
 	printf(_("    %s [OPTIONS] node    {status|check|rejoin|service}\n"), progname());
-	printf(_("    %s [OPTIONS] cluster {show|event|matrix|crosscheck}\n"), progname());
+	printf(_("    %s [OPTIONS] cluster {show|event|matrix|crosscheck|cleanup}\n"), progname());
 	printf(_("    %s [OPTIONS] witness {register|unregister}\n"), progname());
+	printf(_("    %s [OPTIONS] daemon  {status|pause|unpause}\n"), progname());
 
 	puts("");
 
@@ -1894,6 +2048,7 @@ do_help(void)
 	printf(_("  --dry-run                           show what would happen for action, but don't execute it\n"));
 	printf(_("  -L, --log-level                     set log level (overrides configuration file; default: NOTICE)\n"));
 	printf(_("  --log-to-file                       log to file (or logging facility) defined in repmgr.conf\n"));
+	printf(_("  -q, --quiet                         suppress all log output apart from errors\n"));
 	printf(_("  -t, --terse                         don't display detail, hints and other non-critical output\n"));
 	printf(_("  -v, --verbose                       display additional log output (useful for debugging)\n"));
 
@@ -2517,11 +2672,29 @@ remote_command(const char *host, const char *user, const char *command, PQExpBuf
 void
 make_remote_repmgr_path(PQExpBufferData *output_buf, t_node_info *remote_node_record)
 {
+	if (config_file_options.repmgr_bindir[0] != '\0')
+	{
+		int			len = strlen(config_file_options.repmgr_bindir);
+
+		appendPQExpBufferStr(output_buf,
+							 config_file_options.repmgr_bindir);
+
+		/* Add trailing slash */
+		if (config_file_options.repmgr_bindir[len - 1] != '/')
+		{
+			appendPQExpBufferChar(output_buf, '/');
+		}
+	}
+	else if (pg_bindir[0] != '\0')
+	{
+		appendPQExpBufferStr(output_buf,
+							 pg_bindir);
+	}
+
 	appendPQExpBuffer(output_buf,
 					  "%s -f %s ",
-					  make_pg_path(progname()),
+					  progname(),
 					  remote_node_record->config_file);
-
 }
 
 
@@ -2919,3 +3092,45 @@ can_use_pg_rewind(PGconn *conn, const char *data_directory, PQExpBufferData *rea
 
 	return can_use;
 }
+
+
+void
+drop_replication_slot_if_exists(PGconn *conn, int node_id, char *slot_name)
+{
+	t_replication_slot slot_info = T_REPLICATION_SLOT_INITIALIZER;
+	RecordStatus record_status = get_slot_record(conn, slot_name, &slot_info);
+
+	log_verbose(LOG_DEBUG, "attempting to delete slot \"%s\" on node %i",
+				slot_name, node_id);
+
+	if (record_status != RECORD_FOUND)
+	{
+		/* this is a good thing */
+		log_verbose(LOG_INFO,
+					_("slot \"%s\" does not exist on node %i, nothing to remove"),
+					slot_name, node_id);
+	}
+	else
+	{
+		if (slot_info.active == false)
+		{
+			if (drop_replication_slot(conn, slot_name) == true)
+			{
+				log_notice(_("replication slot \"%s\" deleted on node %i"), slot_name, node_id);
+			}
+			else
+			{
+				log_error(_("unable to delete replication slot \"%s\" on node %i"), slot_name, node_id);
+			}
+		}
+
+		/*
+		 * if active replication slot exists, call Houston as we have a
+		 * problem
+		 */
+		else
+		{
+			log_warning(_("replication slot \"%s\" is still active on node %i"), slot_name, node_id);
+		}
+	}
+}
