@@ -35,8 +35,10 @@
 
 static char *config_file = NULL;
 static bool verbose = false;
-static char *pid_file = NULL;
-static bool daemonize = false;
+char pid_file[MAXPGPATH];
+static bool daemonize = true;
+static bool show_pid_file = false;
+static bool no_pid_file = false;
 
 t_configuration_options config_file_options = T_CONFIGURATION_OPTIONS_INITIALIZER;
 
@@ -99,8 +101,11 @@ main(int argc, char **argv)
 		{"config-file", required_argument, NULL, 'f'},
 
 /* daemon options */
-		{"daemonize", no_argument, NULL, 'd'},
+		{"daemonize-short", optional_argument, NULL, 'd'},
+		{"daemonize", optional_argument, NULL, OPT_DAEMONIZE},
 		{"pid-file", required_argument, NULL, 'p'},
+		{"show-pid-file", no_argument, NULL, 's'},
+		{"no-pid-file", no_argument, NULL, OPT_NO_PID_FILE},
 
 /* logging options */
 		{"log-level", required_argument, NULL, 'L'},
@@ -113,8 +118,6 @@ main(int argc, char **argv)
 
 	set_progname(argv[0]);
 
-	srand(time(NULL));
-
 	/* Disallow running as root */
 	if (geteuid() == 0)
 	{
@@ -128,6 +131,10 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	srand(time(NULL));
+
+	memset(pid_file, 0, MAXPGPATH);
+
 	while ((c = getopt_long(argc, argv, "?Vf:L:vdp:m", long_options, &optindex)) != -1)
 	{
 		switch (c)
@@ -172,8 +179,20 @@ main(int argc, char **argv)
 				daemonize = true;
 				break;
 
+			case OPT_DAEMONIZE:
+				daemonize = parse_bool(optarg, "-d/--daemonize", &cli_errors);
+				break;
+
 			case 'p':
-				pid_file = optarg;
+				strncpy(pid_file, optarg, MAXPGPATH);
+				break;
+
+			case 's':
+				show_pid_file = true;
+				break;
+
+			case OPT_NO_PID_FILE:
+				no_pid_file = true;
 				break;
 
 				/* logging options */
@@ -220,7 +239,7 @@ main(int argc, char **argv)
 	/* Exit here already if errors in command line options found */
 	if (cli_errors.head != NULL)
 	{
-		exit_with_cli_errors(&cli_errors);
+		exit_with_cli_errors(&cli_errors, NULL);
 	}
 
 	startup_event_logged = false;
@@ -239,6 +258,58 @@ main(int argc, char **argv)
 	 */
 	load_config(config_file, verbose, false, &config_file_options, argv[0]);
 
+	/* Determine pid file location, unless --no-pid-file supplied */
+
+	if (no_pid_file == false)
+	{
+		if (config_file_options.repmgrd_pid_file[0] != '\0')
+		{
+			if (pid_file[0] != '\0')
+			{
+				log_warning(_("\"repmgrd_pid_file\" will be overridden by --pid-file"));
+			}
+			else
+			{
+				strncpy(pid_file, config_file_options.repmgrd_pid_file, MAXPGPATH);
+			}
+		}
+
+		/* no pid file provided - determine location */
+		if (pid_file[0] == '\0')
+		{
+			/* packagers: if feasible, patch PID file path into "package_pid_file" */
+			char		package_pid_file[MAXPGPATH] = "";
+
+			if (package_pid_file[0] != '\0')
+			{
+				maxpath_snprintf(pid_file, "%s", package_pid_file);
+			}
+			else
+			{
+				const char *tmpdir = getenv("TMPDIR");
+
+				if (!tmpdir)
+					tmpdir = "/tmp";
+
+				maxpath_snprintf(pid_file, "%s/repmgrd.pid", tmpdir);
+			}
+		}
+	}
+	else
+	{
+		/* --no-pid-file supplied - overwrite any value provided with --pid-file ... */
+		memset(pid_file, 0, MAXPGPATH);
+	}
+
+
+	/* If --show-pid-file supplied, output the location (if set) and exit */
+
+	if (show_pid_file == true)
+	{
+		printf("%s\n", pid_file);
+		exit(SUCCESS);
+	}
+
 
 	/* Some configuration file items can be overriden by command line options */
 
@@ -251,8 +322,6 @@ main(int argc, char **argv)
 		strncpy(config_file_options.log_level, cli_log_level, MAXLEN);
 	}
 
-	log_notice(_("repmgrd (repmgr %s) starting up"), REPMGR_VERSION);
-
 	/*
 	 * -m/--monitoring-history, if provided, will override repmgr.conf's
 	 * monitoring_history; this is for backwards compatibility as it's
@@ -280,6 +349,8 @@ main(int argc, char **argv)
 
 	logger_init(&config_file_options, progname());
 
+	log_notice(_("repmgrd (%s %s) starting up"), progname(), REPMGR_VERSION);
+
 	if (verbose)
 		logger_set_verbose();
 
@@ -414,11 +485,14 @@ main(int argc, char **argv)
 		daemonize_process();
 	}
 
-	if (pid_file != NULL)
+	if (pid_file[0] != '\0')
 	{
 		check_and_create_pid_file(pid_file);
 	}
 
+	repmgrd_set_pid(local_conn, getpid(), pid_file);
+
+
 #ifndef WIN32
 	setup_event_handlers();
 #endif
@@ -669,6 +743,8 @@ show_help(void)
 {
 	printf(_("%s: replication management daemon for PostgreSQL\n"), progname());
 	puts("");
+	printf(_("%s monitors a cluster of servers and optionally performs failover.\n"), progname());
+	puts("");
 
 	printf(_("Usage:\n"));
 	printf(_("  %s [OPTIONS]\n"), progname());
@@ -688,19 +764,22 @@ show_help(void)
 
 	puts("");
 
-	printf(_("General configuration options:\n"));
-	printf(_("  -d, --daemonize           detach process from foreground\n"));
-	printf(_("  -p, --pid-file=PATH       write a PID file\n"));
+	printf(_("Daemon configuration options:\n"));
+	printf(_("  -d\n"));
+	printf(_("  --daemonize[=true/false]\n"));
+	printf(_("                            detach process from foreground (default: true)\n"));
+	printf(_("  -p, --pid-file=PATH       use the specified PID file\n"));
+	printf(_("  -s, --show-pid-file       show PID file which would be used by the current configuration\n"));
+	printf(_("  --no-pid-file             don't write a PID file\n"));
 	puts("");
 
-	printf(_("%s monitors a cluster of servers and optionally performs failover.\n"), progname());
 }
 
 
-PGconn *
-try_reconnect(t_node_info *node_info)
+void
+try_reconnect(PGconn **conn, t_node_info *node_info)
 {
-	PGconn	   *conn;
+	PGconn	   *our_conn;
 	t_conninfo_param_list conninfo_params = T_CONNINFO_PARAM_LIST_INITIALIZER;
 
 	int			i;
@@ -709,7 +788,6 @@ try_reconnect(t_node_info *node_info)
 
 	initialize_conninfo_params(&conninfo_params, false);
 
-
 	/* we assume by now the conninfo string is parseable */
 	(void) parse_conninfo_string(node_info->conninfo, &conninfo_params, NULL, false);
 
@@ -732,18 +810,47 @@ try_reconnect(t_node_info *node_info)
 			 * degraded monitoring? - make that configurable
 			 */
 
-			conn = establish_db_connection_by_params(&conninfo_params, false);
+			our_conn = establish_db_connection_by_params(&conninfo_params, false);
 
-			if (PQstatus(conn) == CONNECTION_OK)
+			if (PQstatus(our_conn) == CONNECTION_OK)
 			{
 				free_conninfo_params(&conninfo_params);
 
+				log_info(_("connection to node %i succeeded"), node_info->node_id);
+
+				if (PQstatus(*conn) == CONNECTION_BAD)
+				{
+					log_verbose(LOG_INFO, "original connection handle returned CONNECTION_BAD, using new connection");
+					close_connection(conn);
+					*conn = our_conn;
+				}
+				else
+				{
+					ExecStatusType ping_result;
+
+					ping_result = connection_ping(*conn);
+
+					if (ping_result != PGRES_TUPLES_OK)
+					{
+						log_info("original connnection no longer available, using new connection");
+						close_connection(conn);
+						*conn = our_conn;
+					}
+					else
+					{
+						log_info(_("original connection is still available"));
+
+						PQfinish(our_conn);
+					}
+				}
+
 				node_info->node_status = NODE_STATUS_UP;
-				return conn;
+
+				return;
 			}
 
-			close_connection(&conn);
-			log_notice(_("unable to reconnect to node"));
+			close_connection(&our_conn);
+			log_notice(_("unable to reconnect to node %i"), node_info->node_id);
 		}
 
 		if (i + 1 < max_attempts)
@@ -762,7 +869,7 @@ try_reconnect(t_node_info *node_info)
 
 	free_conninfo_params(&conninfo_params);
 
-	return NULL;
+	return;
 }
 
 
@@ -800,9 +907,12 @@ print_monitoring_state(MonitoringState monitoring_state)
 void
 terminate(int retval)
 {
+	if (PQstatus(local_conn)  == CONNECTION_OK)
+		repmgrd_set_pid(local_conn, UNKNOWN_PID, NULL);
+
 	logger_shutdown();
 
-	if (pid_file)
+	if (pid_file[0] != '\0')
 	{
 		unlink(pid_file);
 	}
